@@ -12,12 +12,14 @@ PY3 = sys.version_info[0] == 3
 
 if PY3:
     import io
+    from pathlib import Path
+
     StringIO = io.StringIO
     BytesIO = io.BytesIO
 
     # Functions for acting on bytestrings and strings. In Python 2 and 3,
     # strings and bytes are the same and chr/ord can be used to convert between
-    # numeric byte values and their string pepresentations. In Python 3, bytes
+    # numeric byte values and their string representations. In Python 3, bytes
     # and strings are different types and bytes hold numeric values when
     # iterated over.
 
@@ -42,8 +44,15 @@ if PY3:
     ifilter = filter
 
     maxint = sys.maxsize
+
+    def path_to_posix(s):
+        return Path(s).as_posix()
+
 else:
     import cStringIO
+    import os
+    import posixpath
+
     StringIO = BytesIO = cStringIO.StringIO
 
     def bytes2hex(b, sep=''):
@@ -63,6 +72,9 @@ else:
 
     maxint = sys.maxint
 
+    def path_to_posix(s):
+        return posixpath.join(*os.path.split(s))
+
 
 def iterkeys(d):
     """Return an iterator over the keys of a dictionary."""
